from __future__ import absolute_import

from analyticsclient.base import BaseEndpoint
from analyticsclient.exceptions import ClientError


class Status(BaseEndpoint):
    """API server status."""

    @property
    def alive(self):
        """
        A very fast shallow check to see if the service is functioning.

        Returns: True iff the remote server responds to requests.

        """
        return self.client.has_resource('status')

    @property
    def authenticated(self):
        """
        Validate the client credentials.

        Returns: True iff the client is successfully authenticated.

        """
        return self.client.has_resource('authenticated')

    @property
    def healthy(self):
        """
        A slow deep health check of the remote service.

        Returns: True iff the remote service is reasonably confident that further operations will succeed.

        """
        try:
            health = self.client.get('health')
        except ClientError:
            return False

        try:
            return health['overall_status'] == 'OK'
        except KeyError:
            return False



NONE = u'none'
OTHER = u'other'
PRIMARY = u'primary'
JUNIOR_SECONDARY = u'junior_secondary'
SECONDARY = u'secondary'
ASSOCIATES = u'associates'
BACHELORS = u'bachelors'
MASTERS = u'masters'
DOCTORATE = u'doctorate'



from __future__ import absolute_import

import six

from analyticsclient.base import BaseEndpoint
from analyticsclient.constants import data_formats


class Programs(BaseEndpoint):
    """Programs client."""

    def programs(self, program_ids=None, fields=None, exclude=None, data_format=data_formats.JSON, **kwargs):
        """
        Get list of programs metadata.

        Arguments:
            program_ids: Array of program IDs as strings to return.  Default is to return all.
            fields: Array of fields to return.  Default is to return all.
            exclude: Array of fields to exclude from response. Default is to not exclude any fields.
        """
        query_params = {}
        for query_arg, data in list(six.moves.zip(['program_ids', 'fields', 'exclude'],
                                                  [program_ids, fields, exclude])) + list(kwargs.items()):
            if data:
                query_params[query_arg] = ','.join(data)

        path = 'programs/'
        querystring = six.moves.urllib.parse.urlencode(query_params)
        if querystring:
            path += '?{0}'.format(querystring)

        return self.client.get(path, data_format=data_format)



from __future__ import absolute_import

import datetime

from analyticsclient.base import PostableCourseIDsEndpoint
from analyticsclient.constants import data_formats


class CourseSummaries(PostableCourseIDsEndpoint):
    """Course summaries."""

    path = 'course_summaries/'

    def course_summaries(
            self,
            course_ids=None,
            availability=None,
            pacing_type=None,
            program_ids=None,
            text_search=None,
            recent_date=None,
            order_by=None,
            sort_order=None,
            page=None,
            page_size=None,
            request_all=False,
            fields=None,
            exclude=None,
            data_format=data_formats.JSON,
    ):
        """
        Get list of summaries.

        For more detailed parameter and return type descriptions, see the
        edX Analytics Data API documentation.

        Arguments:
            course_ids (list[str]): Course IDs to filter by.
            availability (list[str]) Availabilities to filter by.
            pacing_type (list[str]): Pacing types to filter by.
            program_ids (list[str]): Course IDs of programs to filter by.
            text_search (str): Sub-string to search for in course titles and IDs.
            recent_date (date): A date in the past to compute enrollement count
                                change relative to; can be a python date or
                                'YYYY-MM-DD' string
            order_by (str): Summary field to sort by.
            sort_order (str): Order of the sort.
            page (int): Page number.
            page_size (int): Size of page.
            request_all (bool): Whether all summaries should be returned, or just a
                                single page. Overrides `page` and `page_size`.
            fields (list[str]) Fields of course summaries to return in response.
            exclude (list[str]) Fields of course summaries to NOT return in response.
            data_format (str): Data format for response. Must be data_format.JSON or
                               data_format.CSV.

        Returns: dict
        """
        raw_data = {
            'availability': availability,
            'pacing_type': pacing_type,
            'program_ids': program_ids,
            'text_search': text_search,
            'recent_date': recent_date,
            'order_by': order_by,
            'sort_order': sort_order,
            'page': page,
            'page_size': page_size,
            'fields': fields,
            'exclude': exclude,
            'all': request_all,
        }
        data = {
            key: value
            for key, value in raw_data.items()
            if value
        }
        if recent_date and isinstance(recent_date, datetime.date):
            data['recent_date'] = recent_date.strftime('%Y-%m-%d')

        return self.do_request(course_ids=course_ids, data=data, data_format=data_format)



from __future__ import absolute_import

import warnings

import six

from analyticsclient.base import PostableCourseIDsEndpoint
from analyticsclient.constants import activity_types, data_formats
from analyticsclient.exceptions import InvalidRequestError


class Course(PostableCourseIDsEndpoint):
    """Course-related analytics."""

    def __init__(self, client, course_id):
        """
        Initialize the Course client.

        Arguments:

            client (analyticsclient.client.Client): The client to use to access remote resources.
            course_id (str): String identifying the course (e.g. edX/DemoX/Demo_Course)

        """
        super(Course, self).__init__(client)
        self.course_id = six.text_type(course_id)

    def enrollment(self, demographic=None, start_date=None, end_date=None, data_format=data_formats.JSON):
        """
        Get course enrollment data.

        Specify a start or end date to retrieve all data for the date range. If no start or end date is specifying, data
          for the most-recent date will be returned. All dates are in the UTC timezone and should be formatted as
          YYYY-mm-dd (e.g. 2014-01-31).

        Specify a demographic to retrieve data grouped by the specified demographic. If no demographic is specified,
          data will be across all demographics.

        Arguments:
            demographic (str): Demographic by which enrollment data should be grouped.
            start_date (str): Minimum date for returned enrollment data
            end_date (str): Maximum date for returned enrollment data
            data_format (str): Format in which data should be returned
        """
        path = 'courses/{0}/enrollment/'.format(self.course_id)
        if demographic:
            path += '{0}/'.format(demographic)

        params = {}
        if start_date:
            params['start_date'] = start_date

        if end_date:
            params['end_date'] = end_date

        querystring = six.moves.urllib.parse.urlencode(params)
        if querystring:
            path += '?{0}'.format(querystring)

        return self.client.get(path, data_format=data_format)

    def activity(self, activity_type=activity_types.ANY, start_date=None, end_date=None, data_format=data_formats.JSON):
        """
        Get the course student activity.

        Arguments:
            activity_type (str): The type of recent activity to return. Defaults to ANY.
            data_format (str): Format in which data should be returned
        """
        if not activity_type:
            raise InvalidRequestError('activity_type cannot be None.')

        params = {
            'activity_type': activity_type
        }

        if start_date:
            params['start_date'] = start_date

        if end_date:
            params['end_date'] = end_date

        path = 'courses/{0}/activity/'.format(self.course_id)
        querystring = six.moves.urllib.parse.urlencode(params)
        path += '?{0}'.format(querystring)

        return self.client.get(path, data_format=data_format)

    def recent_activity(self, activity_type=activity_types.ANY, data_format=data_formats.JSON):
        """
        Get the recent course activity.

        Arguments:
            activity_type (str): The type of recent activity to return. Defaults to ANY.
            data_format (str): Format in which data should be returned
        """
        warnings.warn('recent_activity has been deprecated! Use activity instead.', DeprecationWarning)

        path = 'courses/{0}/recent_activity/?activity_type={1}'.format(self.course_id, activity_type)
        return self.client.get(path, data_format=data_format)

    def problems(self, data_format=data_formats.JSON):
        """
        Get the problems for the course.

        Arguments:
            data_format (str): Format in which data should be returned
        """
        path = 'courses/{0}/problems/'.format(self.course_id)
        return self.client.get(path, data_format=data_format)

    def problems_and_tags(self, data_format=data_formats.JSON):
        """
        Get the problems for the course with assigned tags.

        Arguments:
            data_format (str): Format in which data should be returned
        """
        path = 'courses/{0}/problems_and_tags/'.format(self.course_id)
        return self.client.get(path, data_format=data_format)

    def reports(self, report_name, data_format=data_formats.JSON):
        """
        Get CSV download information for a particular report in the course.

        Arguments:
            report_name (str): Report name, e.g. "problem_response"
        """
        path = 'courses/{0}/reports/{1}/'.format(self.course_id, report_name)
        return self.client.get(path, data_format=data_format)

    def videos(self, data_format=data_formats.JSON):
        """
        Get the videos for the course.

        Arguments:
            data_format (str): Format in which data should be returned
        """
        path = 'courses/{0}/videos/'.format(self.course_id)
        return self.client.get(path, data_format=data_format)



from __future__ import absolute_import

import six

from analyticsclient.base import PostableCourseIDsEndpoint
from analyticsclient.constants import data_formats


class EngagementTimeline(PostableCourseIDsEndpoint):
    """Engagement Timeline."""

    def __init__(self, client, username, course_id):
        """
        Initialize the EngagementTimeline client.

        Arguments:

            client (analyticsclient.client.Client): The client to use to access remote resources.
            username (str): String identifying the user (e.g. jbradley)
            course_id (str): String identifying the course (e.g. edX/DemoX/Demo_Course)

        """
        super(EngagementTimeline, self).__init__(client)

        self.username = six.text_type(username)
        self.course_id = six.text_type(course_id)

    def get(self):
        """Get a particular learner's engagement timeline for a particular course."""
        querystring = six.moves.urllib.parse.urlencode({'course_id': self.course_id})
        path = 'engagement_timelines/{username}/?{querystring}'.format(username=self.username, querystring=querystring)
        return self.client.get(path, data_format=data_formats.JSON)



from __future__ import absolute_import

import six

from analyticsclient.base import BaseEndpoint
from analyticsclient.constants import data_formats


class Module(BaseEndpoint):
    """Module related analytics data."""

    def __init__(self, client, course_id, module_id):
        """
        Initialize the Module client.

        Arguments:
            client (analyticsclient.client.Client): The client to use to access the API.
            course_id (str): String identifying the course
            module_id (str): String identifying the module
        """
        super(Module, self).__init__(client)
        self.course_id = six.text_type(course_id)
        self.module_id = six.text_type(module_id)

    def answer_distribution(self, data_format=data_formats.JSON):
        """
        Get answer distribution data for a module.

        Arguments:
            data_format (str): Format in which to return data (default is JSON)
        """
        path = 'problems/{0}/answer_distribution/'.format(self.module_id)

        return self.client.get(path, data_format=data_format)

    def grade_distribution(self, data_format=data_formats.JSON):
        """
        Get grade distribution data for a module.

        Arguments:
            data_format (str): Format in which to return data (default is JSON)
        """
        path = 'problems/{0}/grade_distribution/'.format(self.module_id)

        return self.client.get(path, data_format=data_format)

    def sequential_open_distribution(self, data_format=data_formats.JSON):
        """
        Get open distribution data for a module.

        Arguments:
            data_format (str): Format in which to return data (default is JSON)
        """
        path = 'problems/{0}/sequential_open_distribution/'.format(self.module_id)

        return self.client.get(path, data_format=data_format)

    def video_timeline(self, data_format=data_formats.JSON):
        """
        Get video segments/timeline for a module.

        Arguments:
            data_format (str): Format in which to return data (default is JSON)
        """
        path = 'videos/{0}/timeline/'.format(self.module_id)

        return self.client.get(path, data_format=data_format)



from __future__ import absolute_import

from analyticsclient.base import PostableCourseIDsEndpoint
from analyticsclient.constants import data_formats


class CourseTotals(PostableCourseIDsEndpoint):
    """Course aggregate data."""

    path = 'course_totals/'

    def course_totals(self, course_ids=None, data_format=data_formats.JSON):
        """
        Get aggregate data about courses.

        For more detailed parameter and return type descriptions, see the
        edX Analytics Data API documentation.

        Arguments:
            course_ids (list[str]): Course IDs to filter by.
            data_format (str): Data format for response.
                               Must be data_format.JSON or data_format.CSV.
        """
        return self.do_request(course_ids=course_ids, data={}, data_format=data_format)



from __future__ import absolute_import

from analyticsclient.constants import data_formats, http_methods


class BaseEndpoint:
    """Base class for endpoints that use a client object."""

    def __init__(self, client):
        """
        Initialize the API client.

        Arguments:
            client (analyticsclient.client.Client): The client to use to access remote resources.
        """
        self.client = client


class PostableCourseIDsEndpoint(BaseEndpoint):
    """Base class for endpoints that pass in course IDs with either GET or POST."""

    path = None  # Override in subclass
    max_num_ids_for_get = 10  # Optionally override in subclass

    def do_request(self, course_ids, data, data_format=data_formats.JSON):
        """
        Given course IDs, do the appropriate request method (GET or POST).

        Arguments:
            course_ids (list[str]): A list of course IDs to pass in
            data (dict): Arguments for endpoint, sans course IDs
            data_format (data_format)

        Returns: dict
        """
        data_with_ids = data.copy()
        if course_ids:
            data_with_ids.update(course_ids=course_ids)
        method = (
            http_methods.POST
            if len(course_ids or []) > self.max_num_ids_for_get
            else http_methods.GET
        )
        return self.client.request(method, self.path, data=data_with_ids, data_format=data_format)



from __future__ import absolute_import, print_function

import codecs
import os
import sys
from setuptools import setup

from analyticsclient import __version__ as VERSION


if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m '%s'" % (VERSION, VERSION))
    os.system("git push --tags")
    sys.exit()

with codecs.open('README.rst', 'r', 'utf-8') as f:
    LONG_DESCRIPTION = f.read()

def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns a list of requirement strings.
    """
    requirements = set()
    for path in requirements_paths:
        with open(path) as reqs:
            requirements.update(
                line.split('#')[0].strip() for line in reqs
                if is_requirement(line.strip())
            )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement;
    that is, it is not blank, a comment, a URL, or an included file.
    """
    return line and not line.startswith(('-r', '#', '-e', 'git+', '-c'))


setup(
    name='edx-analytics-data-api-client',
    version=VERSION,
    packages=['analyticsclient', 'analyticsclient.constants'],
    url='https://github.com/edx/edx-analytics-data-api-client',
    description='Client used to access edX analytics data warehouse',
    long_description=LONG_DESCRIPTION,
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.8',
    ],
    install_requires=load_requirements('requirements/base.in'),
    test_requires=load_requirements('requirements/base.in')
)



class ClientError(Exception):
    """Common base class for all client errors."""


class NotFoundError(ClientError):
    """URL was not found."""


class InvalidRequestError(ClientError):
    """The API request was invalid."""


class TimeoutError(ClientError):  # pylint: disable=redefined-builtin
    """The API server did not respond before the timeout expired."""



from __future__ import absolute_import

import logging

import requests
import requests.exceptions

from analyticsclient.constants import data_formats, http_methods
from analyticsclient.course import Course
from analyticsclient.course_summaries import CourseSummaries
from analyticsclient.course_totals import CourseTotals
from analyticsclient.engagement_timeline import EngagementTimeline
from analyticsclient.exceptions import ClientError, InvalidRequestError, NotFoundError, TimeoutError  # pylint: disable=redefined-builtin
from analyticsclient.module import Module
from analyticsclient.programs import Programs
from analyticsclient.status import Status

log = logging.getLogger(__name__)


class Client:
    """
    Analytics API client.

    The instance has attributes `status` and `courses` that provide access to instances of
    :class: `~analyticsclient.status` and :class: `~analyticsclient.course`. This is the preferred (and only supported)
    way to get access to those classes and their methods.
    """

    # Date/time formats to be used when sending and parsing data from the API
    DATE_FORMAT = '%Y-%m-%d'
    DATETIME_FORMAT = DATE_FORMAT + 'T%H%M%S'

    def __init__(self, base_url, auth_token=None, timeout=0.25):
        """
        Initialize the client.

        Arguments:
            base_url (str): URL of the API server (e.g. http://analytics.edx.org/api/v1)
            auth_token (str): Authentication token
            timeout (number): Maximum number of seconds during which all requests musts complete
        """
        self.base_url = base_url.rstrip('/')
        self.auth_token = auth_token
        self.timeout = timeout

        self.status = Status(self)
        self.course_summaries = lambda: CourseSummaries(self)
        self.course_totals = lambda: CourseTotals(self)
        self.programs = lambda: Programs(self)
        self.courses = lambda course_id: Course(self, course_id)
        self.modules = lambda course_id, module_id: Module(self, course_id, module_id)
        self.engagement_timeline = lambda username, course_id: EngagementTimeline(self, username, course_id)

    def get(self, *args, **kwargs):
        """
        Retrieve the data for a resource.

        Equivalent to `request(http_methods.GET, ...)`

        Returns: API response data in specified data_format

        Raises: ClientError if the resource cannot be retrieved for any reason.
        """
        return self.request(http_methods.GET, *args, **kwargs)

    def request(self, method, resource, data=None, timeout=None, data_format=data_formats.JSON):
        """
        Retrieve the from an HTTP request.

        Arguments:

            method (http_method): HTTP method. Only GET and POST are supported currenly.
            resource (str): Path in the form of slash separated strings.
            data (dict): Dictionary containing POST data.
            timeout (float): Continue to attempt to retrieve a resource for this many seconds before giving up and
                raising an error.
            data_format (str): Format in which data should be returned

        Returns: API response data in specified data_format

        Raises: ClientError if the resource cannot be retrieved for any reason.

        """
        response = self._request(
            method=method,
            resource=resource,
            data=data,
            timeout=timeout,
            data_format=data_format
        )

        if data_format == data_formats.CSV:
            return response.text

        try:
            return response.json()
        except ValueError:
            message = 'Unable to decode JSON response'
            log.exception(message)
            raise ClientError(message)

    def has_resource(self, resource, timeout=None):
        """
        Check if the server responds with a 200 OK status code when the resource is requested.

        Inherited from `Client`.

        Arguments:

            resource (str): Path in the form of slash separated strings.
            timeout (float): Continue to attempt to retrieve a resource for this many seconds before giving up and
                raising an error.

        Returns: True iff the resource exists.

        """
        try:
            self._request(http_methods.GET, resource, timeout=timeout)
            return True
        except ClientError:
            return False

    # pylint: disable=no-member
    def _request(self, method, resource, data=None, timeout=None, data_format=data_formats.JSON):
        if timeout is None:
            timeout = self.timeout

        accept_format = 'application/json'
        if data_format == data_formats.CSV:
            accept_format = 'text/csv'

        headers = {
            'Accept': accept_format,
        }

        if self.auth_token:
            headers['Authorization'] = 'Token ' + self.auth_token

        try:
            uri = '{0}/{1}'.format(self.base_url, resource)

            if method == http_methods.GET:
                params = self._data_to_get_params(data or {})
                response = requests.get(uri, params=params, headers=headers, timeout=timeout)
            elif method == http_methods.POST:
                response = requests.post(uri, data=(data or {}), headers=headers, timeout=timeout)
            else:
                raise ValueError(
                    'Invalid \'method\' argument: expected {0} or {1}, got {2}'.format(
                        http_methods.GET,
                        http_methods.POST,
                        method,
                    )
                )

            status = response.status_code
            if status != requests.codes.ok:
                message = 'Resource "{0}" returned status code {1}'.format(resource, status)
                error_class = ClientError

                if status == requests.codes.bad_request:
                    message = 'The request to {0} was invalid.'.format(uri)
                    error_class = InvalidRequestError
                elif status == requests.codes.not_found:
                    message = 'Resource {0} was not found on the API server.'.format(uri)
                    error_class = NotFoundError

                log.error(message)
                raise error_class(message)

            return response

        except requests.exceptions.Timeout:
            message = "Response from {0} exceeded timeout of {1}s.".format(resource, timeout)
            log.exception(message)
            raise TimeoutError(message)

        except requests.exceptions.RequestException:
            message = 'Unable to retrieve resource'
            log.exception(message)
            raise ClientError('{0} "{1}"'.format(message, resource))

    @staticmethod
    def _data_to_get_params(data):
        return {
            key: (
                ','.join(value)
                if isinstance(value, list)
                else str(value)
            )
            for key, value in data.items()
        }
